@@ -28,7 +28,9 @@ from neutron.callbacks import exceptions as callbacks_exc
 from neutron.callbacks import registry
 from neutron.callbacks import resources
 import neutron.common.exceptions as ml2_exceptions
+from neutron.conf.plugins.ml2 import config  # noqa
 from neutron.db import _resource_extend as resource_extend
+from neutron.db import agents_db
 from neutron.db import api as q_db_api
 from neutron.db.availability_zone import router as router_az
 from neutron.db import db_base_plugin_v2
@@ -43,16 +45,20 @@ from neutron.db import l3_dvr_db
 from neutron.db import l3_hamode_db  # noqa
 from neutron.db import models_v2
 from neutron.db import portbindings_db
-from neutron.extensions import availability_zone as az_ext
-from neutron.extensions import external_net
-from neutron.extensions import l3
 from neutron.extensions import providernet as provider
+from neutron.objects.qos import policy as policy_object
+from neutron.plugins.ml2 import managers as n_managers
+from neutron_lib.api.definitions import availability_zone as az_def
+from neutron_lib.api.definitions import external_net
+from neutron_lib.api.definitions import l3 as l3_apidef
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net
 from neutron_lib.api import validators
+from neutron_lib.api.validators import availability_zone as az_validator
 import neutron_lib.callbacks.resources as attributes
 from neutron_lib import constants
 from neutron_lib import exceptions
+from neutron_lib.exceptions import availability_zone as az_exc
 from neutron_lib.plugins import directory
 import neutronclient.common.exceptions as q_cli_exceptions
 
@@ -70,6 +76,7 @@ from tricircle.db import models
 import tricircle.network.exceptions as t_network_exc
 from tricircle.network import helper
 from tricircle.network import managers
+from tricircle.network import qos_driver
 from tricircle.network import security_groups
 
 
@@ -78,6 +85,11 @@ tricircle_opts = [
                 default=['vxlan,local'],
                 help=_('List of network type driver entry points to be loaded '
                        'from the tricircle.network.type_drivers namespace.')),
+    cfg.ListOpt('extension_drivers',
+                default=[],
+                help=_('List of network extension driver entry points to be '
+                       'loaded from the neutron.ml2.extension_drivers '
+                       'namespace.')),
     cfg.ListOpt('tenant_network_types',
                 default=['vxlan,local'],
                 help=_('Ordered list of network_types to allocate as tenant '
@@ -126,6 +138,7 @@ NON_VM_PORT_TYPES = [constants.DEVICE_OWNER_ROUTER_INTF,
 
 
 class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
+                      agents_db.AgentDbMixin,
                       security_groups.TricircleSecurityGroupMixin,
                       external_net_db.External_net_db_mixin,
                       portbindings_db.PortBindingMixin,
@@ -163,12 +176,15 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
     def __init__(self):
         super(TricirclePlugin, self).__init__()
         LOG.info("Starting Tricircle Neutron Plugin")
-        self.clients = {}
+        self.clients = {'top': t_client.Client()}
         self.xjob_handler = xrpcapi.XJobAPI()
         self._setup_rpc()
         self.type_manager = managers.TricircleTypeManager()
+        self.extension_manager = n_managers.ExtensionManager()
+        self.extension_manager.initialize()
         self.type_manager.initialize()
         self.helper = helper.NetworkHelper(self)
+        qos_driver.register()
 
     def _setup_rpc(self):
         self.endpoints = []
@@ -199,7 +215,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         self._validate_availability_zones(context, availability_zones)
 
     def get_router_availability_zones(self, router_db):
-        return router_db.get(az_ext.AZ_HINTS)
+        return router_db.get(az_def.AZ_HINTS)
 
     @staticmethod
     def _validate_availability_zones(context, az_list):
@@ -216,14 +232,14 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
 
             diff = az_set - known_az_set
             if diff:
-                raise az_ext.AvailabilityZoneNotFound(
+                raise az_exc.AvailabilityZoneNotFound(
                     availability_zone=diff.pop())
 
     @staticmethod
     @resource_extend.extends([attributes.NETWORKS])
     def _extend_availability_zone(net_res, net_db):
-        net_res[az_ext.AZ_HINTS] = az_ext.convert_az_string_to_list(
-            net_db[az_ext.AZ_HINTS])
+        net_res[az_def.AZ_HINTS] = az_validator.convert_az_string_to_list(
+            net_db[az_def.AZ_HINTS])
 
     @staticmethod
     def _ensure_az_set_for_external_network(context, req_data):
@@ -231,10 +247,10 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         external_set = validators.is_attr_set(external)
         if not external_set or not external:
             return False
-        if az_ext.AZ_HINTS in req_data and req_data[az_ext.AZ_HINTS]:
+        if az_def.AZ_HINTS in req_data and req_data[az_def.AZ_HINTS]:
             return True
         # if no az_hints are specified, we will use default region_name
-        req_data[az_ext.AZ_HINTS] = \
+        req_data[az_def.AZ_HINTS] = \
             [cfg.CONF.tricircle.default_region_for_external_network]
         return True
 
@@ -248,7 +264,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
     def _create_bottom_external_network(self, context, net, top_id):
         t_ctx = t_context.get_context_from_neutron_context(context)
         # use the first pod
-        az_name = net[az_ext.AZ_HINTS][0]
+        az_name = net[az_def.AZ_HINTS][0]
         pod = db_api.find_pod_by_az_or_region(t_ctx, az_name)
         body = {
             'network': {
@@ -265,7 +281,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
 
     def _create_bottom_external_subnet(self, context, subnet, net, top_id):
         t_ctx = t_context.get_context_from_neutron_context(context)
-        region_name = net[az_ext.AZ_HINTS][0]
+        region_name = net[az_def.AZ_HINTS][0]
         pod = db_api.get_pod_by_name(t_ctx, region_name)
         b_net_id = db_api.get_bottom_id_by_top_id_region_name(
             t_ctx, net['id'], region_name, t_constants.RT_NETWORK)
@@ -294,23 +310,25 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         tenant_id = net_data['tenant_id']
         is_external = self._ensure_az_set_for_external_network(context,
                                                                net_data)
-        if az_ext.AZ_HINTS in net_data:
+        if az_def.AZ_HINTS in net_data:
             self._validate_availability_zones(context,
-                                              net_data[az_ext.AZ_HINTS])
+                                              net_data[az_def.AZ_HINTS])
         with q_db_api.context_manager.writer.using(context):
             net_db = self.create_network_db(context, network)
             res = self._make_network_dict(net_db, process_extensions=False,
                                           context=context)
+            self.extension_manager.process_create_network(context, net_data,
+                                                          res)
             self._process_l3_create(context, res, net_data)
             net_data['id'] = res['id']
             self.type_manager.create_network_segments(context, net_data,
                                                       tenant_id)
             self.type_manager.extend_network_dict_provider(context, res)
-            if az_ext.AZ_HINTS in net_data:
-                az_hints = az_ext.convert_az_list_to_string(
-                    net_data[az_ext.AZ_HINTS])
-                net_db[az_ext.AZ_HINTS] = az_hints
-                res[az_ext.AZ_HINTS] = net_data[az_ext.AZ_HINTS]
+            if az_def.AZ_HINTS in net_data:
+                az_hints = az_validator.convert_az_list_to_string(
+                    net_data[az_def.AZ_HINTS])
+                net_db[az_def.AZ_HINTS] = az_hints
+                res[az_def.AZ_HINTS] = net_data[az_def.AZ_HINTS]
             # put inside a session so when bottom operations fails db can
             # rollback
             if is_external:
@@ -390,33 +408,74 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         self._raise_if_updates_external_attribute(net_data)
 
         with context.session.begin():
-            net = super(TricirclePlugin, self).update_network(context,
-                                                              network_id,
-                                                              network)
+            original_network = super(TricirclePlugin, self).get_network(
+                context, network_id)
+            policy = policy_object.QosPolicy.get_network_policy(
+                context, network_id)
+            if policy:
+                original_network['qos_policy_id'] = policy['id']
+            else:
+                original_network['qos_policy_id'] = None
+
+            updated_network = super(
+                TricirclePlugin, self).update_network(
+                context, network_id, network)
+
+            self.extension_manager.process_update_network(
+                context, net_data, updated_network)
+
+            self.type_manager.extend_network_dict_provider(context,
+                                                           updated_network)
+
+            updated_network = self.get_network(context, network_id)
+
+            if net_data.get('qos_policy_id', None):
+                updated_network['qos_policy_id'] = net_data['qos_policy_id']
+
+            if not updated_network.get('qos_policy_id', None):
+                updated_network['qos_policy_id'] = None
+
+            need_network_update_notify = (
+                'qos_policy_id' in net_data and
+                original_network['qos_policy_id'] !=
+                updated_network['qos_policy_id'])
+
             t_ctx = t_context.get_context_from_neutron_context(context)
             mappings = db_api.get_bottom_mappings_by_top_id(
                 t_ctx, network_id, t_constants.RT_NETWORK)
             if mappings:
                 self.xjob_handler.update_network(
-                    t_ctx, net['tenant_id'], network_id,
+                    t_ctx, updated_network['tenant_id'], network_id,
                     t_constants.POD_NOT_SPECIFIED)
 
-            self.type_manager.extend_network_dict_provider(context, net)
-            return net
+            if need_network_update_notify and \
+                    updated_network['qos_policy_id'] and mappings:
+                t_policy_id = updated_network['qos_policy_id']
+                self.xjob_handler.create_qos_policy(
+                    t_ctx, updated_network['tenant_id'], t_policy_id,
+                    t_constants.POD_NOT_SPECIFIED, t_constants.RT_NETWORK,
+                    updated_network['id'])
+
+            return updated_network
 
     def _convert_az2region_for_nets(self, context, nets):
         for net in nets:
             self._convert_az2region_for_net(context, net)
 
     def _convert_az2region_for_net(self, context, net):
-        az_hints = net.get(az_ext.AZ_HINTS)
+        az_hints = net.get(az_def.AZ_HINTS)
         if context.is_admin and az_hints:
             t_ctx = t_context.get_context_from_neutron_context(context)
-            net[az_ext.AZ_HINTS] = self._convert_az2region(t_ctx, az_hints)
+            net[az_def.AZ_HINTS] = self._convert_az2region(t_ctx, az_hints)
 
     def _convert_az2region(self, t_ctx, az_hints):
         return self.helper.convert_az2region(t_ctx, az_hints)
 
+    def _get_network_qos_info(self, context, net_id):
+        policy = policy_object.QosPolicy.get_network_policy(
+            context.elevated(), net_id)
+        return policy['id'] if policy else None
+
     def get_network(self, context, network_id, fields=None):
         net = super(TricirclePlugin, self).get_network(context, network_id,
                                                        fields)
@@ -425,6 +484,9 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
 
         self._convert_az2region_for_net(context, net)
 
+        net['qos_policy_id'] = \
+            self._get_network_qos_info(context.elevated(), net['id'])
+
         return net
 
     def get_networks(self, context, filters=None, fields=None,
@@ -436,6 +498,11 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             self.type_manager.extend_networks_dict_provider(context, nets)
 
         self._convert_az2region_for_nets(context, nets)
+
+        for net in nets:
+            net['qos_policy_id'] = \
+                self._get_network_qos_info(context.elevated(), net['id'])
+
         return nets
 
     def create_subnet(self, context, subnet):
@@ -559,6 +626,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         self._ensure_default_security_group_on_port(context, port)
         sgids = self._get_security_groups_on_port(context, port)
         result = self._make_port_dict(db_port)
+        self.extension_manager.process_create_port(context, port_body, result)
         self._process_port_create_security_group(context, result, sgids)
         return result
 
@@ -669,7 +737,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
     def update_port(self, context, port_id, port):
         t_ctx = t_context.get_context_from_neutron_context(context)
         top_port = super(TricirclePlugin, self).get_port(context, port_id)
-
+        updated_port = None
         # be careful that l3_db will call update_port to update device_id of
         # router interface, we cannot directly update bottom port in this case,
         # otherwise we will fail when attaching bottom port to bottom router
@@ -677,15 +745,17 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         if t_constants.PROFILE_REGION in port['port'].get(
                 'binding:profile', {}):
             # this update request comes from local Neutron
-            res = super(TricirclePlugin, self).update_port(context, port_id,
-                                                           port)
+            updated_port = super(TricirclePlugin, self).update_port(context,
+                                                                    port_id,
+                                                                    port)
+
             profile_dict = port['port']['binding:profile']
             region_name = profile_dict[t_constants.PROFILE_REGION]
             device_name = profile_dict[t_constants.PROFILE_DEVICE]
             t_ctx = t_context.get_context_from_neutron_context(context)
             pod = db_api.get_pod_by_name(t_ctx, region_name)
 
-            net = self.get_network(context, res['network_id'])
+            net = self.get_network(context, updated_port['network_id'])
             is_vxlan_network = (
                 net[provider_net.NETWORK_TYPE] == t_constants.NT_VxLAN)
             if is_vxlan_network:
@@ -698,20 +768,41 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
                 # gateway port, but we only need to create resource routing
                 # entries, trigger xjob and configure security group rules for
                 # instance port
-                self._create_mapping_for_vm_port(t_ctx, res, pod)
+                self._create_mapping_for_vm_port(t_ctx, updated_port, pod)
                 self._process_trunk_port(context, t_ctx,
-                                         res, pod, profile_dict)
+                                         updated_port, pod, profile_dict)
                 # only trigger setup_bottom_router job
-                self._trigger_router_xjob_for_vm_port(context, res, pod)
+                self._trigger_router_xjob_for_vm_port(context, updated_port,
+                                                      pod)
                 self.xjob_handler.configure_security_group_rules(
-                    t_ctx, res['tenant_id'])
+                    t_ctx, updated_port['tenant_id'])
 
             if is_vxlan_network and (
                     cfg.CONF.client.cross_pod_vxlan_mode in (
                         t_constants.NM_P2P, t_constants.NM_L2GW)):
-                self.xjob_handler.setup_shadow_ports(t_ctx, res['tenant_id'],
-                                                     pod['pod_id'],
-                                                     res['network_id'])
+                self.xjob_handler.setup_shadow_ports(
+                    t_ctx, updated_port['tenant_id'], pod['pod_id'],
+                    updated_port['network_id'])
+
+            network_binding_policy = \
+                policy_object.QosPolicy.get_network_policy(
+                    context, updated_port['network_id'])
+
+            port_binding_policy = policy_object.QosPolicy.get_port_policy(
+                context, port_id)
+
+            if network_binding_policy:
+                t_policy_id = network_binding_policy['id']
+                self.xjob_handler.create_qos_policy(
+                    t_ctx, t_ctx.project_id, t_policy_id, pod['pod_id'],
+                    t_constants.RT_NETWORK, updated_port['network_id'])
+
+            if port_binding_policy:
+                t_policy_id = port_binding_policy['id']
+                self.xjob_handler.create_qos_policy(
+                    t_ctx, t_ctx.project_id, t_policy_id, pod['pod_id'],
+                    t_constants.RT_PORT, port_id)
+
         # for vm port or port with empty device_owner, update top port and
         # bottom port
         elif top_port.get('device_owner') not in NON_VM_PORT_TYPES:
@@ -720,6 +811,9 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             request_body = port[attributes.PORT]
             if mappings:
                 with context.session.begin():
+                    original_qos_policy_id = \
+                        self._get_port_qos_info(context, port_id)
+
                     b_pod, b_port_id = mappings[0]
                     b_region_name = b_pod['region_name']
                     b_client = self._get_client(region_name=b_region_name)
@@ -731,36 +825,58 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
                         self._handle_bottom_security_group(
                             t_ctx, request_body['security_groups'], b_pod)
 
-                    res = super(TricirclePlugin, self).update_port(
+                    updated_port = super(TricirclePlugin, self).update_port(
                         context, port_id, port)
+                    self.extension_manager.process_update_port(
+                        context, request_body, updated_port)
+                    updated_port = \
+                        super(TricirclePlugin, self).get_port(context, port_id)
                     # name is not allowed to be updated, because it is used by
                     # lock_handle to retrieve bottom/local resources that have
                     # been created but not registered in the resource routing
                     # table
                     request_body.pop('name', None)
 
-                    try:
-                        b_client.update_ports(t_ctx, b_port_id, port)
-                    except q_cli_exceptions.NotFound:
-                        LOG.error(
-                            ('port: %(port_id)s not found, '
-                             'region name: %(name)s'),
-                            {'port_id': b_port_id, 'name': b_region_name})
+                    request_body_policy_id = \
+                        request_body.get('qos_policy_id', None)
+                    if request_body_policy_id:
+                            request_body.pop('qos_policy_id')
+
+                    if request_body:
+                        try:
+                            b_client.update_ports(t_ctx, b_port_id, port)
+                        except q_cli_exceptions.NotFound:
+                            LOG.error(
+                                ('port: %(port_id)s not found, '
+                                 'region name: %(name)s'),
+                                {'port_id': b_port_id, 'name': b_region_name})
 
                     if request_body.get('security_groups', None):
                         self.xjob_handler.configure_security_group_rules(
-                            t_ctx, res['tenant_id'])
+                            t_ctx, updated_port['tenant_id'])
+
+                    updated_port['qos_policy_id'] = request_body_policy_id
+                    if request_body_policy_id and \
+                        original_qos_policy_id != \
+                            request_body_policy_id:
+                        t_policy_id = updated_port['qos_policy_id']
+                        self.xjob_handler.create_qos_policy(
+                            t_ctx, t_ctx.project_id,
+                            t_policy_id, b_pod['pod_id'],
+                            t_constants.RT_PORT, b_port_id)
             else:
                 self._filter_unsupported_attrs(request_body)
-                res = super(TricirclePlugin, self).update_port(
+                updated_port = super(TricirclePlugin, self).update_port(
                     context, port_id, port)
+                self.extension_manager.process_update_port(
+                    context, request_body, updated_port)
         else:
             # for router interface, router gw, dhcp port, not directly
             # update bottom port
-            res = super(TricirclePlugin, self).update_port(
+            updated_port = super(TricirclePlugin, self).update_port(
                 context, port_id, port)
         self._log_update_port_sensitive_attrs(port_id, port)
-        return res
+        return updated_port
 
     def _pre_delete_port(self, context, port_id, port_check):
         """Do some preliminary operations before deleting the port."""
@@ -814,6 +930,10 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
                                                 'value': port_id}])
         super(TricirclePlugin, self).delete_port(context, port_id)
 
+    def _get_port_qos_info(self, context, port_id):
+        policy = policy_object.QosPolicy.get_port_policy(context, port_id)
+        return policy['id'] if policy else None
+
     def get_port(self, context, port_id, fields=None):
         t_ctx = t_context.get_context_from_neutron_context(context)
         mappings = db_api.get_bottom_mappings_by_top_id(
@@ -828,28 +948,30 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             if fields:
                 port = dict(
                     [(k, v) for k, v in six.iteritems(port) if k in fields])
-            if 'network_id' not in port and 'fixed_ips' not in port:
-                return port
-
-            bottom_top_map = {}
-            with t_ctx.session.begin():
-                for resource in (t_constants.RT_SUBNET, t_constants.RT_NETWORK,
-                                 t_constants.RT_ROUTER):
-                    route_filters = [{'key': 'resource_type',
-                                      'comparator': 'eq',
-                                      'value': resource}]
-                    routes = core.query_resource(
-                        t_ctx, models.ResourceRouting, route_filters, [])
-                    for route in routes:
-                        if route['bottom_id']:
-                            bottom_top_map[
-                                route['bottom_id']] = route['top_id']
-            self._map_port_from_bottom_to_top(port, bottom_top_map)
-            return port
+            if 'network_id' in port or 'fixed_ips' in port:
+                bottom_top_map = {}
+                with t_ctx.session.begin():
+                    for resource in (t_constants.RT_SUBNET,
+                                     t_constants.RT_NETWORK,
+                                     t_constants.RT_ROUTER):
+                        route_filters = [{'key': 'resource_type',
+                                          'comparator': 'eq',
+                                          'value': resource}]
+                        routes = core.query_resource(
+                            t_ctx, models.ResourceRouting, route_filters, [])
+                        for route in routes:
+                            if route['bottom_id']:
+                                bottom_top_map[
+                                    route['bottom_id']] = route['top_id']
+                self._map_port_from_bottom_to_top(port, bottom_top_map)
         else:
-            return super(TricirclePlugin, self).get_port(context,
+            port = super(TricirclePlugin, self).get_port(context,
                                                          port_id, fields)
 
+        port['qos_policy_id'] = \
+            self._get_port_qos_info(context, port_id)
+        return port
+
     @staticmethod
     def _apply_ports_filters(query, model, filters):
         if not filters:
@@ -1049,8 +1171,13 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         if not filters or 'id' not in filters:
             # if filter is empty or "id" is not in the filter, no special
             # handle is required
-            return self._get_ports(context, filters, fields, sorts, limit,
-                                   marker, page_reverse)
+            ports = self._get_ports(context, filters, fields, sorts, limit,
+                                    marker, page_reverse)
+            for port in ports:
+                port['qos_policy_id'] = \
+                    self._get_port_qos_info(context, port['id'])
+
+            return ports
         if len(filters) == 1:
             # only "id" is in the filter, we use get_port to get all the ports
             ports = []
@@ -1066,9 +1193,14 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             id_filters = filters.pop('id')
             ports = self._get_ports(context, filters, None, sorts, limit,
                                     marker, page_reverse)
-            return [super(TricirclePlugin,
-                          self)._fields(
+            ports = [super(TricirclePlugin,
+                           self)._fields(
                 p, fields) for p in ports if p['id'] in id_filters]
+            for port in ports:
+                port['qos_policy_id'] = \
+                    self._get_port_qos_info(context, port['id'])
+
+            return ports
 
     def _get_ports(self, context, filters=None, fields=None, sorts=None,
                    limit=None, marker=None, page_reverse=False):
@@ -1433,7 +1565,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
 
     def _add_router_gateway(self, context, router_id, router_data):
         # get top external network information
-        ext_net_id = router_data[l3.EXTERNAL_GW_INFO].get('network_id')
+        ext_net_id = router_data[l3_apidef.EXTERNAL_GW_INFO].get('network_id')
         t_ctx = t_context.get_context_from_neutron_context(context)
         network = self.get_network(context, ext_net_id)
 
@@ -1441,14 +1573,14 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         # az hint parameter, so tricircle plugin knows where to create the
         # corresponding bottom external network. here we get bottom external
         # network ID from resource routing table.
-        if not network.get(az_ext.AZ_HINTS):
+        if not network.get(az_def.AZ_HINTS):
             raise t_exceptions.ExternalNetPodNotSpecify()
 
         t_router = self._get_router(context, router_id)
         self.validate_router_net_location_match(t_ctx, t_router, network)
         is_local_router = self.helper.is_local_router(t_ctx, t_router)
 
-        region_name = network[az_ext.AZ_HINTS][0]
+        region_name = network[az_def.AZ_HINTS][0]
         pod = db_api.get_pod_by_name(t_ctx, region_name)
         b_net_id = db_api.get_bottom_id_by_top_id_region_name(
             t_ctx, ext_net_id, region_name, t_constants.RT_NETWORK)
@@ -1464,10 +1596,6 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
                                'distributed': is_distributed}}
 
         else:
-            # TODO(zhiyuan) decide router is distributed or not from pod table
-            # currently "distributed" is set to False, should add a metadata
-            # field to pod table, and decide distributed or not from the
-            # metadata later
             router_type = t_constants.RT_NS_ROUTER
             body = {'router': {'name': t_constants.ns_router_name % router_id,
                                'distributed': False}}
@@ -1478,7 +1606,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         # both router and external network in bottom pod are ready, attach
         # external network to router in bottom pod.
         b_client = self._get_client(region_name)
-        t_info = router_data[l3.EXTERNAL_GW_INFO]
+        t_info = router_data[l3_apidef.EXTERNAL_GW_INFO]
         b_info = {'network_id': b_net_id}
         if 'enable_snat' in t_info:
             b_info['enable_snat'] = t_info['enable_snat']
@@ -1544,12 +1672,12 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             return
         ext_net_id = gw_port['network_id']
         t_network = self.get_network(context, ext_net_id)
-        if az_ext.AZ_HINTS not in t_network:
+        if az_def.AZ_HINTS not in t_network:
             raise t_exceptions.ExternalNetPodNotSpecify()
-        if not t_network[az_ext.AZ_HINTS]:
+        if not t_network[az_def.AZ_HINTS]:
             raise t_exceptions.ExternalNetPodNotSpecify()
 
-        region_name = t_network[az_ext.AZ_HINTS][0]
+        region_name = t_network[az_def.AZ_HINTS][0]
         is_local_router = self.helper.is_local_router(t_ctx, t_router)
 
         b_router_id = db_api.get_bottom_id_by_top_id_region_name(
@@ -1570,29 +1698,20 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         router_data = copy.deepcopy(router['router'])
         need_update_bottom = False
         is_add = False
-        if validators.is_attr_set(router_data.get(l3.EXTERNAL_GW_INFO)):
+        if validators.is_attr_set(router_data.get(l3_apidef.EXTERNAL_GW_INFO)):
             need_update_bottom = True
-            ext_net_id = router_data[l3.EXTERNAL_GW_INFO].get('network_id')
+            ext_net_id = router_data[l3_apidef.EXTERNAL_GW_INFO].get(
+                'network_id')
             if ext_net_id:
                 is_add = True
-        # TODO(zhiyuan) solve ip address conflict issue
-        # if user creates floating ip before set router gateway, we may trigger
-        # ip address conflict here. let's say external cidr is 163.3.124.0/24,
-        # creating floating ip before setting router gateway, the gateway ip
-        # will be 163.3.124.3 since 163.3.124.2 is used by floating ip, however
-        # in the bottom pod floating ip is not created when creating floating
-        # ip on top, so the gateway ip in the bottom pod is still 163.3.124.2,
-        # thus conflict may occur.
-        #
-        # before this issue is solved, user should set router gateway before
-        # create floating ip.
         if not need_update_bottom:
             return super(TricirclePlugin, self).update_router(
                 context, router_id, router)
         if is_add:
             ret = super(TricirclePlugin, self).update_router(
                 context, router_id, router)
-            router_data[l3.EXTERNAL_GW_INFO].update(ret[l3.EXTERNAL_GW_INFO])
+            router_data[l3_apidef.EXTERNAL_GW_INFO].update(
+                ret[l3_apidef.EXTERNAL_GW_INFO])
             self._add_router_gateway(context, router_id, router_data)
         else:
             self._remove_router_gateway(context, router_id)
@@ -1613,7 +1732,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         router_region_names = self._convert_az2region(t_ctx, router_az_hints)
         router_region_set = set(router_region_names)
 
-        net_az_hints = net.get(az_ext.AZ_HINTS)
+        net_az_hints = net.get(az_def.AZ_HINTS)
         if not net_az_hints:
             if is_local_router:
                 # network az hints parameter is not specified, meaning that
