@@ -18,15 +18,16 @@ import six
 from oslo_config import cfg
 from oslo_log import log
 
+from neutron_lib.api.definitions import availability_zone as az_def
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net
 from neutron_lib.api import validators
 import neutron_lib.constants as q_constants
 import neutron_lib.exceptions as q_exceptions
 from neutron_lib.plugins import directory
+from neutron_lib.utils import runtime
 
 from neutron.common import utils
-from neutron.extensions import availability_zone as az_ext
 import neutron.extensions.securitygroup as ext_sg
 from neutron.plugins.ml2 import plugin
 
@@ -68,7 +69,7 @@ class TricirclePlugin(plugin.Ml2Plugin):
         super(TricirclePlugin, self).__init__()
         core_plugins_namespace = 'neutron.core_plugins'
         plugin_provider = cfg.CONF.tricircle.real_core_plugin
-        plugin_class = utils.load_class_by_alias_or_classname(
+        plugin_class = runtime.load_class_by_alias_or_classname(
             core_plugins_namespace, plugin_provider)
         self.core_plugin = plugin_class()
         self.neutron_handle = resource_handle.NeutronResourceHandle(
@@ -244,12 +245,39 @@ class TricirclePlugin(plugin.Ml2Plugin):
                                               net_body['name'])
             if net_id:
                 net_body['id'] = net_id
+
+        net_body.pop('qos_policy_id', None)
+
         b_network = self.core_plugin.create_network(context,
                                                     {'network': net_body})
         return b_network
 
+    def _is_valid_network(self, context, network_id):
+        try:
+            self.core_plugin.get_network(context, network_id)
+        except q_exceptions.NotFound:
+            if self._in_subnet_delete(context):
+                raise
+            t_ctx = t_context.get_context_from_neutron_context(context)
+
+            t_network = self.neutron_handle.handle_get(
+                t_ctx, 'network', network_id)
+            region_name = self._get_neutron_region()
+            located = self._is_network_located_in_region(t_network,
+                                                         region_name)
+            if not located:
+                LOG.error('network: %(network_id)s not located in current '
+                          'region: %(region_name)s, '
+                          'az_hints: %(az_hints)s',
+                          {'network_id': t_network['id'],
+                           'region_name': region_name,
+                           'az_hints': t_network[az_def.AZ_HINTS]})
+                return located
+            self._create_bottom_network(context, network_id)
+        return True
+
     def _is_network_located_in_region(self, t_network, region_name):
-        az_hints = t_network.get(az_ext.AZ_HINTS)
+        az_hints = t_network.get(az_def.AZ_HINTS)
         if not az_hints:
             return True
         return region_name in az_hints
@@ -267,12 +295,7 @@ class TricirclePlugin(plugin.Ml2Plugin):
             t_ctx = t_context.get_context_from_neutron_context(context)
             if self._skip_non_api_query(t_ctx):
                 raise q_exceptions.NetworkNotFound(net_id=_id)
-            t_network = self.neutron_handle.handle_get(t_ctx, 'network', _id)
-            if not t_network:
-                raise q_exceptions.NetworkNotFound(net_id=_id)
-            self._adapt_network_body(t_network)
-            b_network = self.core_plugin.create_network(context,
-                                                        {'network': t_network})
+            t_network, b_network = self._create_bottom_network(context, _id)
             subnet_ids = self._ensure_subnet(context, t_network)
         if subnet_ids:
             b_network['subnets'] = subnet_ids
@@ -325,10 +348,12 @@ class TricirclePlugin(plugin.Ml2Plugin):
                               'az_hints: %(az_hints)s',
                               {'net_id': network['id'],
                                'region_name': region_name,
-                               'az_hints': network[az_ext.AZ_HINTS]})
+                               'az_hints': network[az_def.AZ_HINTS]})
                     continue
 
                 self._adapt_network_body(network)
+
+                network.pop('qos_policy_id', None)
                 b_network = self.core_plugin.create_network(
                     context, {'network': network})
                 subnet_ids = self._ensure_subnet(context, network)
@@ -362,10 +387,20 @@ class TricirclePlugin(plugin.Ml2Plugin):
             b_gateway_ip)['subnet']
         t_subnet['gateway_ip'] = subnet_body['gateway_ip']
         t_subnet['allocation_pools'] = subnet_body['allocation_pools']
-
         b_subnet = self.core_plugin.create_subnet(q_ctx, {'subnet': t_subnet})
         return b_subnet
 
+    def _create_bottom_network(self, context, _id):
+        t_ctx = t_context.get_context_from_neutron_context(context)
+        t_network = self.neutron_handle.handle_get(t_ctx, 'network', _id)
+        if not t_network:
+            raise q_exceptions.NetworkNotFound(net_id=_id)
+        self._adapt_network_body(t_network)
+        t_network.pop('qos_policy_id', None)
+        b_network = self.core_plugin.create_network(context,
+                                                    {'network': t_network})
+        return t_network, b_network
+
     def get_subnet(self, context, _id, fields=None):
         t_ctx = t_context.get_context_from_neutron_context(context)
         try:
@@ -376,6 +411,9 @@ class TricirclePlugin(plugin.Ml2Plugin):
             t_subnet = self.neutron_handle.handle_get(t_ctx, 'subnet', _id)
             if not t_subnet:
                 raise q_exceptions.SubnetNotFound(subnet_id=_id)
+            valid = self._is_valid_network(context, t_subnet['network_id'])
+            if not valid:
+                raise q_exceptions.SubnetNotFound(subnet_id=_id)
             b_subnet = self._create_bottom_subnet(t_ctx, context, t_subnet)
         if b_subnet['enable_dhcp']:
             self._ensure_subnet_dhcp_port(t_ctx, context, b_subnet)
@@ -416,6 +454,9 @@ class TricirclePlugin(plugin.Ml2Plugin):
             missing_subnets = [subnet for subnet in t_subnets if (
                 subnet['id'] in missing_id_set)]
             for subnet in missing_subnets:
+                valid = self._is_valid_network(context, subnet['network_id'])
+                if not valid:
+                    continue
                 b_subnet = self._create_bottom_subnet(t_ctx, context, subnet)
                 if b_subnet['enable_dhcp']:
                     self._ensure_subnet_dhcp_port(t_ctx, context, b_subnet)
@@ -559,6 +600,8 @@ class TricirclePlugin(plugin.Ml2Plugin):
 
         self._handle_security_group(t_ctx, context, t_port)
         self._create_shadow_agent(context, port_body)
+
+        t_port.pop('qos_policy_id', None)
         b_port = self.core_plugin.create_port(context, {'port': t_port})
         return b_port
 
@@ -744,6 +787,7 @@ class TricirclePlugin(plugin.Ml2Plugin):
             self._ensure_network_subnet(context, t_port)
             self._adapt_port_body_for_call(t_port)
             self._handle_security_group(t_ctx, context, t_port)
+            t_port.pop('qos_policy_id', None)
             b_port = self.core_plugin.create_port(context, {'port': t_port})
 
         self._ensure_trunk(context, t_ctx, _id)
@@ -787,6 +831,7 @@ class TricirclePlugin(plugin.Ml2Plugin):
             self._ensure_network_subnet(context, port)
             self._adapt_port_body_for_call(port)
             self._handle_security_group(t_ctx, context, port)
+            port.pop('qos_policy_id', None)
             b_port = self.core_plugin.create_port(context,
                                                   {'port': port})
             b_ports.append(self._fields(b_port, fields))
