@@ -138,8 +138,8 @@ class FakeClient(test_utils.FakeClient):
             key = query_filter['key']
             # when querying trunks, "fields" is passed in the query string to
             # ask the server to only return necessary fields, which can reduce
-            # the data being transfered. in test, we just return all the fields
-            # since there's no need to optimize
+            # the data being transferred. In test, we just return all the
+            # fields since there's no need to optimize
             if key != 'fields':
                 value = query_filter['value']
                 filter_dict[key] = value
@@ -529,12 +529,12 @@ class PluginTest(unittest.TestCase):
                                                      update_body)
         self.assertEqual(updated_top_trunk['name'], 'new_name')
         self.assertEqual(updated_top_trunk['description'], 'updated')
-        self.assertEqual(updated_top_trunk['admin_state_up'], False)
+        self.assertFalse(updated_top_trunk['admin_state_up'])
 
         updated_btm_trunk = fake_plugin.get_trunk(q_ctx, t_trunk['id'])
         self.assertEqual(updated_btm_trunk['name'], 'new_name')
         self.assertEqual(updated_btm_trunk['description'], 'updated')
-        self.assertEqual(updated_btm_trunk['admin_state_up'], False)
+        self.assertFalse(updated_btm_trunk['admin_state_up'])
 
     @patch.object(context, 'get_context_from_neutron_context',
                   new=fake_get_context_from_neutron_context)
